import { ElementType } from './MultiArray';
export type MathObject = ElementType;
export type UnaryMathOperation = (expression: MathObject) => MathObject;
export type BinaryMathOperation = (left: MathObject, right: MathObject) => MathObject;
export type MathOperationType = UnaryMathOperation | BinaryMathOperation;
export declare abstract class MathOperation {
    static unaryOperations: {
        [name: string]: UnaryMathOperation;
    };
    static binaryOperations: {
        [name: string]: BinaryMathOperation;
    };
    static leftAssociativeMultipleOperations: {
        [name: string]: BinaryMathOperation;
    };
    static copy(right: MathObject): MathObject;
    private static elementWiseOperation;
    private static leftOperation;
    static plus(left: MathObject, right: MathObject): MathObject;
    static minus(left: MathObject, right: MathObject): MathObject;
    static mod(left: MathObject, right: MathObject): MathObject;
    static rem(left: MathObject, right: MathObject): MathObject;
    static times(left: MathObject, right: MathObject): MathObject;
    static mtimes(left: MathObject, right: MathObject): MathObject;
    static rdivide(left: MathObject, right: MathObject): MathObject;
    static mrdivide(left: MathObject, right: MathObject): MathObject;
    static ldivide(left: MathObject, right: MathObject): MathObject;
    static mldivide(left: MathObject, right: MathObject): MathObject;
    static power(left: MathObject, right: MathObject): MathObject;
    static mpower(left: MathObject, right: MathObject): MathObject;
    static uplus(right: MathObject): MathObject;
    static uminus(right: MathObject): MathObject;
    static transpose(left: MathObject): MathObject;
    static ctranspose(left: MathObject): MathObject;
    static lt(left: MathObject, right: MathObject): MathObject;
    static le(left: MathObject, right: MathObject): MathObject;
    static eq(left: MathObject, right: MathObject): MathObject;
    static ge(left: MathObject, right: MathObject): MathObject;
    static gt(left: MathObject, right: MathObject): MathObject;
    static ne(left: MathObject, right: MathObject): MathObject;
    static mand(left: MathObject, right: MathObject): MathObject;
    static mor(left: MathObject, right: MathObject): MathObject;
    static not(right: MathObject): MathObject;
    static and(left: MathObject, right: MathObject): MathObject;
    static or(left: MathObject, right: MathObject): MathObject;
    static xor(left: MathObject, right: MathObject): MathObject;
}
