import { ComplexDecimal } from './ComplexDecimal';
import { Evaluator } from './Evaluator';
import { ElementType, MultiArray } from './MultiArray';
export declare class Structure {
    static readonly STRUCTURE: number;
    readonly type: number;
    parent: any;
    field: Record<string, ElementType>;
    private static readonly invalidReferenceMessage;
    /**
     * Structure constructor. If an object is passed as parameter then create
     * a Structure with same fields and values of object. If an array of field
     * names as string is passed then create a Structure with this field
     * branch and nested field value set to empty array.
     * @param field An object with fields and values or an array of field names.
     */
    constructor(field: Record<string, ElementType> | string[]);
    /**
     *
     * @param S
     * @param field
     * @param value
     */
    static setField(S: Structure, field: string[], value?: ElementType): void;
    /**
     *
     * @param S
     * @param field
     * @param value
     */
    static setNewField(S: Structure, field: string[], value?: ElementType): void;
    /**
     *
     * @param obj
     * @param field
     * @returns
     */
    static getField(obj: ElementType, field: string[]): ElementType;
    static getFields(obj: ElementType, field: string[]): ElementType[];
    /**
     *
     * @param S
     * @param evaluator
     * @returns
     */
    static unparse(S: Structure, evaluator: Evaluator): string;
    /**
     *
     * @param S
     * @param evaluator
     * @returns
     */
    static unparseMathML(S: Structure, evaluator: Evaluator): string;
    /**
     *
     * @param S
     * @returns
     */
    static copy(S: Structure): Structure;
    /**
     *
     * @returns
     */
    copy(): Structure;
    /**
     *
     * @param S
     * @returns
     */
    static cloneFields(S: Structure): Structure;
    /**
     *
     * @param S
     * @returns
     */
    static toLogical(S: Structure): ComplexDecimal;
    /**
     *
     * @returns
     */
    toLogical(): ComplexDecimal;
    /**
     * Set empty field in all elements of MultiArray if it is not cell array and if field not defined.
     * @param M
     * @param field
     */
    static setEmptyField(M: MultiArray, field: string): void;
}
